%diffuse
% A particle is placed at (0,0,0) in a cylindrical tube which extends from
% x=-a to x = b, with radius r. The particle makes a random walk, and
% collides elastically with the walls of the tube. This program works out
% the trajectory, and determines how long it takes to make it out of the
% tube. The left hand end is also an elastic wall.

function diffuse

%Number of trajectories
N = 100;

%Define tube parameters /m
a = -1; b = 10; r = 2;

%Define a fixed particle speed /ms^-1
s = 10;

%Define timestep /s
dt = 0.01;

%Define maximum simulation time /s
tmax = 100;

%Graph fontsize
fsize = 18;

%%

%Initialize the plot
figure('renderer','opengl');
plot3(0,0,0,'g*'); hold on;
xx = [];
for n=1:N
    %Work out trajectories and store x positions
    [x,y,z,t] = diffusion_trajectory(a,r,s,dt,tmax);
    xx = [xx;x];
    
    %Truncate values if x > b
    i = x>b;
    x(i) = []; y(i) = []; z(i) = [];
    
    %Plot trajectories
    plot3( x,y,z,'color',rand(1,3) );
    plot3( x(end),y(end),z(end),'r*' );

end
xlabel('x','fontsize', fsize ); ylabel('y','fontsize', fsize );...
zlabel('z','fontsize', fsize );
axis equal; axis vis3d; grid on; box on;
set( gca, 'fontsize', fsize );
xlim([a,b]);
ylim([-r,r]);
zlim([-r,r]);
title(['Diffusion. Particle speed =',num2str(s),'m/s'], 'fontsize', fsize );
print( gcf,'diffusion.png','-r300','-dpng' )
close(gcf);

%Determine average x displacement
x = mean( xx, 1 );
x(x<0) = 0;

%Plot mean x positions vs time
figure;
plot( t, x );
axis tight;
grid on;
set( gca, 'fontsize', fsize );
xlabel('t /s', 'fontsize', fsize )
ylabel('Mean distance diffused /m', 'fontsize', fsize )
title(['x vs t for diffusion, v = ',num2str(s),'m/s'], 'fontsize', fsize )
print( gcf,'diffusion mean x vs t.png','-r300','-dpng' )
close(gcf);

%Determine power law for x = A*t^B
figure;
plot( log(t), log(x) ); hold on;
axis tight
grid on;
set( gca, 'fontsize', fsize );
xlabel('log(t/s)', 'fontsize', fsize )
ylabel('log( Mean distance diffused /m)', 'fontsize', fsize )
[yfit,xfit,r,ybar,xbar,m,c] = bestfitc( log(t),log(x) );
plot( xfit, yfit, 'k-' );
title({['Mean x vs t for diffusion, v = ',num2str(s),'m/s'],...
    ['x = ',num2str(exp(c),3),'t^{',num2str(m,3),'}']}, 'fontsize', fsize );
print( gcf,'diffusion power law.png','-r300','-dpng' )
close(gcf);

%%

%Determine trajectory
function [x,y,z,t] = diffusion_trajectory(a,r,s,dt,tmax)

%Initialize x,y,z,t coordinates
x = 0; y = 0; z = 0; t = 0;

%Compute the random walk!
n=1;
while t(end) < tmax
    
    %Choose random direction angles
    azi = 2*pi*rand; elev = -0.5*pi + pi*rand;
    
    %Define magnitude of displacement during timestep
    d = s*dt;
    
    %Compute next x,y position
    dx = d*cos(elev)*cos(azi);
    dy = d*cos(elev)*sin(azi);
    dz = d*sin(elev);
    
    %Check if particle has hit a side wall
    if ( ( y(end)+dy )^2 + ( z(end) + dz )^2 ) > r^2
        dz = -dz; dy = -dy;
    end
    
    %Check if particle has hit back wall
    if ( x(end) + dx ) < a
        dx = -dx;
    end
    
    %Update x,y,z trajectory
    x(n+1) = x(n) + dx; y(n+1) = y(n) + dy; z(n+1) = z(n) + dz;
    t(n+1) = t(n) + dt; n = n+1;
end

%%

%Line of best fit function yfit = m*x +c, with product moment correlation
%coefficient r
function [yfit,xfit,r,ybar,xbar,m,c] = bestfitc(x,y)

%Find any x or y values that are NaN or Inf
ignore = isnan(abs(x)) | isnan(abs(y)) | isinf(abs(x)) | isinf(abs(y));
x(ignore) = [];
y(ignore) = [];

%Compute line of best fit
xbar = mean(x);
ybar = mean(y);
xybar = mean(x.*y);
Vx = mean(x.^2 ) - xbar^2;
Vy = mean(y.^2 ) - ybar^2;
COVxy = mean(x.*y) - xbar*ybar;
m = COVxy/Vx;
c = ybar - m*xbar;
r = COVxy/sqrt( Vx*Vy );
yfit = m*x + c;
xfit = x;

%End of code
