%gravity_sim_1_star_and_planet
% Simulates the circular orbit of a planet around a massive star
%
% LAST UPDATED by Andy French March 2019

function gravity_sim_1_star_and_planet

num_periods = 7; %Number of orbital periods
a = 5.202; %Semi-major axis of orbit in AU
M = 1.0; %Mass of star in solar masses
dt = 0.01; %Timestep in years
fsize = 18; %Fontsize

%Determine period in years via Kepler III
T = sqrt( (1/M)*a^3 );

%Set initial x,y,vx,vy,t parameters
x = a; y = 0; vx = 0; vy = 2*pi*sqrt( M/a ); t = 0;

%Determine initial accelerations from Newton II
[ax,ay] = gravity( x,y, 0, 0, M );

%Now determine orbit using Verlet method
n=1;
while t(end) < num_periods * T
    t(n+1) = t(n) + dt;
    x(n+1) = x(n) + vx(n)*dt + 0.5*ax*dt^2;
    y(n+1) = y(n) + vy(n)*dt + 0.5*ay*dt^2;
    ax_old = ax; ay_old = ay;
    [ax,ay] = gravity( x(n+1),y(n+1), 0, 0, M );
    vx(n+1) = vx(n) + 0.5*dt*( ax_old + ax );
    vy(n+1) = vy(n) + 0.5*dt*( ay_old + ay );
    n = n + 1;
end

%Plot trajectory
figure('color',[1 1 1],'name','star and planet');
plot(x,y,'b'); hold on
plot( 0,0,'r*','markersize',20 );
p = plot(x(1),y(1),'b*','markersize',10);
axis equal; axis manual;
grid on; set(gca,'fontsize',fsize);
xlabel('x /AU','fontsize',fsize); ylabel('y /AU','fontsize',fsize);
title({'Star and planet',...
    ['M=',num2str(M),' solar masses, T=',num2str(T),' years']},...
    'fontsize',fsize)

%Animation
stop = 0;
N = length(x);
n=1;
while stop == 0
    set( p, 'xdata', x(n), 'ydata', y(n) );
    drawnow;
    n = n+1;
    if n>N
        n=1;
    end
end

%%

%Newton's law of gravitation. x,y accelerations of a mass at location x,y
%resulting from the force of gravity of mass M1 at location x1,y1.
%Units of distance are AU and time are years.
function [ax,ay] = gravity( x,y, x1, y1, M1 )
ax = -( 4*pi^2 )*M1*(x-x1)/ ( ( (x-x1)^2 + (y-y1)^2 )^(3/2) ) ;
ay = -( 4*pi^2 )*M1*(y-y1)/ ( ( (x-x1)^2 + (y-y1)^2 )^(3/2) ) ;

%End of code


