%make_julia_map
% Function that greates a Julia map. This is an iterative function of
% complex variable z=x+i*y where x and y are the real and imaginary numbers
% respectively. A bitmap image of the map is saved to the current
% working directory.
%
% Syntax:
% julia_map=make_julia_map(width,height,max_pixels,xwidth,xc,yc,N,funcstr,..
% map_function,map_creation_rule,C,NaN_colour,cmap_str,min_julia,max_julia,isnorm,clim)
%
% width, height  - Size of bitmap image in pixels
% max_pixels     - Define maximum number of pixel in the X or Y dimensions.
%                  If an image has a dimension greater than this, several
%                  Julia maps will be formed and then stitched together.
%                  If this is not done then large images (> 2000 x 2000)
%                  may not be possible to form due to the large amount
%                  of memory required.
% xwidth         - X width (Y width is computed using bitmap image aspect ratio)
% xc,yc          - Coordinates of plot centre
% N              - Number of iterations
% funcstr        - String of julia function of z,z0 and index.
%                  i.e. 'z.^2 +z0(index)'
% z              - Matrix of complex numbers
% z0             - Matrix of complex numbers, same size as z
%                  z0 is typically the coordinates describing the argand diagram
%                  upon which z is plotted.
% index          - Matrix of size of z which gives the index of the elements of z
%                  relative to the original map z0. Note z and index may be smaller
%                  in size than z0
% map_function   - Julia map function
%                  Options: 'exp_decay' 'sin' 'cos' 'arg'
%                  otherwise 'abs' is used
% map_creation_rule
%                - Julia map creation rule
%                  Options: 'plot z' 'abs diverge', 'abs diverge & mod(2)=0'
%                  're and im diverge' 're or im diverge'
% C              - z convergence radius
% NaN_colour     - [R,G,B] colour of NaN regions of the map
% cmap_str       - Colormap string i.e. 'prism'
% min_julia      - Min value of Julia map real values
% max_julia      - Max value of Julia map real values
% isnorm         - Binary flag. If == 1 then map is created for color
%                  normalization purposes. The waitbar text will refelect this.
% clim           - Vector describing range of normalized julia map that correspond to
%                  the colormap
% dont_patch     - If this ==1 then don't attempt to tile the julia map
% no_bitmap      - If this ==1 the don't create a bitmap
%
% julia_map      - Normalized (0...1) Julia map matrix of dimensions width x height.

function [julia_map,x,y]=make_julia_map(width,height,max_pixels,xwidth,xc,yc,N,funcstr,...
    map_function,map_creation_rule,C,NaN_colour,cmap_str,min_julia,max_julia,isnorm,clim,dont_patch,no_bitmap)

%% CREATE JUIA MAP %%

%Turn off warnings
warning off;

%Set up waitbar
n=0;
frac_complete=n/N;
wait_bar=waitbar(frac_complete,['Computing Julia map ... ',...
    num2str(floor(100*frac_complete)),'% complete'] );
if isnorm==1
    set(wait_bar,'name','Computing max and min magnitudes of julia map');
else
    set(wait_bar,'name','Computing julia map')
end

%Define patches
if dont_patch == 1
    
    %Set maximum of height and width to correspond to max_pixels
    if height>width
        new_height = max_pixels;
        new_width = round( width*max_pixels/height );
    else
        new_width = max_pixels;
        new_height = round( height*max_pixels/width );
    end
    width= new_width;
    height = new_height;
    
    %Split up the width
    num_x = 1;
    num_y = 1;
else
    %Split up the width and height into patches
    num_x = ceil(width / max_pixels);
    num_y = ceil(height / max_pixels);
end
rem_x = mod(width, max_pixels);
rem_y = mod(height, max_pixels);

%Define y width
ywidth=xwidth*height/width;

%Create Argand diagram mesh
x=linspace(xc-0.5*xwidth,xc+0.5*xwidth,width);
y=linspace(yc-0.5*ywidth,yc+0.5*ywidth,height);

%Dividing argand diagram into small patches (real and imaginary dimensions
%must be less than max_pixels) compute separate julia maps and write these
%to the current MATLAB directory as indexed bitmap files julia_map_1.png,
%julia_map_2.png etc.
num = 0;
for nx = 1:num_x
    for ny = 1:num_y
        
        %Increment patch number
        num = num + 1;
        
        %Define section of the Argand diagram to be processed
        
        %Indices of initially defined Argand diagram
        if ((nx < num_x) || (rem_x == 0))
            indx = ((nx-1)*max_pixels + 1):(nx*max_pixels);
        else
            indx = ((nx-1)*max_pixels + 1):((nx-1)*max_pixels + rem_x);
        end
        
        if ((ny < num_y) || (rem_y == 0))
            indy = ((ny-1)*max_pixels + 1):(ny*max_pixels);
        else
            indy = ((ny-1)*max_pixels + 1):((ny-1)*max_pixels + rem_y);
        end
        
        %Define subset of initially defined Argand diagram
        xn = x(indx);
        yn = y(indy);
        [xn,yn]=meshgrid(xn,yn);
        
        %Compute new Argand diagram
        z = xn + 1i*yn;
        z0 = z;
        
        %Compute new widths
        new_width = length(indx);
        new_height = length(indy);
        
        %Initialise index matrix (numbered columnwise)
        L_new = new_width * new_height;
        index = uint32(1:new_width * new_height);
        index = reshape(index, [new_height, new_width]);
        
        %Compute julia map and update wait bar
        
        %%% PLOT Z %%%
        if strcmp(map_creation_rule,'plot z')==1
            
            %Don't use divergence tests, simply plot the Nth iteration
            %of z, when transformed using julia_map_function
            for n=1:N
                
                %Get data from GUI and check if STOP flag has been set
                handles = guidata(findobj('tag','FIGUREjulia'));
                if handles.J.STOP == 1
                    julia_map = [];
                    close(wait_bar);
                    return
                end
                
                %Apply julia function
                z=juliafunction(z,z0,index,funcstr);
                if isempty(z)
                    julia_map = [];
                    close(wait_bar);
                    return
                end
                frac_complete=n/N;
                waitbar(frac_complete,wait_bar,...
                    ['Computing Julia map ... ',...
                    num2str(floor(100*frac_complete)),'% complete (',...
                    num2str(num),' of ',num2str(num_x*num_y),')']);
            end
            julia_map=julia_map_function(z,map_function);
        else
            
            %%% DIVERGENCE TEST JULIA MAP %%%%
            
            %Initialise divergence iteration matrix
            div_iter=NaN(size(z));
            
            %Initialise other matricies
            fail_indicies=uint32(zeros(size(z)));
            div_test_indices=uint32(zeros(size(z)));
            test_matrix = NaN(size(z));
            
            %Iterate tests and compute julia map
            n=1;
            while (L_new>0) && (n <=N)
                
                %Get data from GUI and check if STOP flag has been set
                handles = guidata(findobj('tag','FIGUREjulia'));
                if handles.J.STOP == 1
                    julia_map = [];
                    close(wait_bar);
                    return
                end
                
                %Apply julia function
                z=juliafunction(z,z0,index,funcstr);
                if isempty(z)
                    julia_map = [];
                    close(wait_bar);
                    return
                end
                
                %Use divergence rules to compute iteration n what each
                %pixel passes divergence test
                if strcmp(map_creation_rule,'re and im diverge')==1
                    test_matrix=(abs(real(z))>C) && (abs(imag(z))>C);
                elseif strcmp(map_creation_rule,'re or im diverge')==1
                    test_matrix=(abs(real(z))>C) | (abs(imag(z))>C);
                elseif strcmp(map_creation_rule,'abs diverge & mod(2)=0')==1
                    test_matrix=(abs(z)>C) && (mod(n,2)==0);
                else
                    test_matrix=abs(z)>C;
                end
                
                %Determine indices that pass test
                div_test_indices=find(test_matrix==1);
                fail_indices=find(test_matrix==0);
                
                %Compute new length of z
                L_new=prod(size(fail_indices));
                
                %Assign iteration number to these indices of div_iter
                div_iter(index(div_test_indices))=n;
                
                %Remove elements that pass test from z and index
                z=z(fail_indices);
                index=index(fail_indices);
                
                %Update waitbar
                frac_complete=n/N;
                waitbar(frac_complete,wait_bar,...
                    ['Computing Julia map ... ',...
                    num2str(floor(100*frac_complete)),'% complete (',...
                    num2str(num),' of ',num2str(num_x*num_y),')']);
                
                %Update n
                n=n+1;
            end
            
            clear div_test_indices;
            clear fail_indicies;
            clear test_matrix;
            clear index;
            clear z;
            
            %Normalize by dividing map by the number of iterations
            div_iter=div_iter./ N;
            
            %Apply julia map function
            julia_map=julia_map_function(div_iter,map_function);
        end
        
        %Normalise real part of Julia map i.e. to range 0...1
        julia_map=real(julia_map);
        if ~isempty(min_julia) && ~isempty(max_julia)
            julia_map =( julia_map -min_julia*ones(size(julia_map)) ) / (max_julia-min_julia);
        end
        
        %Save julia map directly to a TIFF file
        if no_bitmap ~=1
            waitbar(frac_complete,wait_bar,['Writing Julia map bitmap ',...
                num2str(num),'... ']);
            
            %Create temporary figure and obtain colormap
            temp_fig = figure; axes;
            if strcmp(cmap_str,'black & white')==1
                %Create alternating black & white colormap
                cmap=zeros(N,3);
                for k=1:N
                    if mod(k,2)==0
                        cmap(k,:)=[1,1,1];
                    end
                end
            else
                cmap=colormap(gca,cmap_str);
            end
            close(temp_fig);
            
            %Construct colour matrix corresponding to Julia map and write PNG
            %file for this composite of the total map
            julia_colours=make_colour_matrix(julia_map,cmap,NaN_colour,clim);
            imwrite(julia_colours,['julia map iter=',leadingzero(N,3),' ',num2str(num),'.png'],'png');
        end
    end
end

%Close wait bar
close(wait_bar);

%End of code

