%logistic
% Plots an orbit diagram for a one-dimensional logistic map on the interval
% [0,1], demonstrating 'May's Bifircation diagram' as discussed in
% Strogatz pp353+
%
% LAST UPDATED by Andy French Jun 2017

function logistic
global d

%%%% INPUTS %%%%

%Define number of iterations to start plotting, and number of plots
d.N = 2000;

%Define number of values of growth parameter r
d.M = 2000;

%Define initial start and end range of growth parameter r
d.rstart = 0;
d.rend = 5;

%Plot fontsize
d.fsize = 16;

%Plot marker size
d.msize = 0.1;

%

%%%% CREATE FIGURE %%%%

%Define vector of growth parameters
r = linspace( d.rstart, d.rend, d.M );

%Compute orbit diagram
[x,rr] = may( r, d.N );

%Plot the orbit diagram r,x
d.fig = figure('color',[1 1 1],'name','Logistic map - PRESS U to update P to print PNG T to toggle probability map','KeyPressFcn',@keypress);
d.p=plot( rr, x, 'k.');
set(d.p,'markersize',d.msize);
set(gca,'fontsize',d.fsize)
xlabel('Growth parameter r','fontsize',d.fsize)
ylabel('x','fontsize',d.fsize)
title('May Bifurcations Logistic map','fontsize',d.fsize)
xlim( [d.rstart,d.rend]);
axis manual
box on;
grid on;

%%

%Key press handler. Save .png
%file "logistic map.png" if 'p' key is pressed. Otherwise update figure
function keypress( fig_handle,evnt )
global d

%Get the string of the current key press
k = get( d.fig,'currentkey');

%Actions, depending on the key pressed...
if strcmp(k,'p') == 1
    print(d.fig,'logistic map.png','-dpng','-r300')
elseif strcmp(k,'t') == 1
    
    %Get axis limits
    xlimits = get(gca,'xlim');
    ylimits = get(gca,'ylim');
    
    %Define vector of growth parameters
    r = linspace( xlimits(1), xlimits(2), d.M );
    
    %Compute orbit diagram
    [x,rr] = may( r, d.N );
    
    %Determine probability map, based upon 500 bins between x=0 and x=1 for
    %each growth parameter
    BINS = 1000;
    rr = repmat( r, [BINS, 1]);
    xx = repmat( linspace(ylimits(1),ylimits(2),BINS).',[1,d.N] );
    pmap = log10( hist( x ,linspace(ylimits(1),ylimits(2),BINS))/d.N );
    
    %Update plot
    delete(d.p);
    d.p = pcolor( rr, xx , pmap );
    interp_colormap(BINS)
    shading interp
    colorbar
    set(d.p,'markersize',d.msize);
    set(gca,'fontsize',d.fsize)
    xlabel('Growth parameter r','fontsize',d.fsize)
    ylabel('x','fontsize',d.fsize)
    title('May Bifurcations Logistic map probability','fontsize',d.fsize)
    xlim( [xlimits(1),xlimits(2)]);
    ylim( [ylimits(1),ylimits(2)]);
    axis manual
    box on;
    grid on;
    
else
    %Get axis limits
    xlimits = get(gca,'xlim');
    ylimits = get(gca,'ylim');
    
    %Define vector of growth parameters
    r = linspace( xlimits(1), xlimits(2), d.M );
    
    %Compute orbit diagram
    [x,rr] = may( r, d.N );
    
    %Update plot
    delete(d.p);
    d.p=plot( rr, x, 'k.');
    set(d.p,'markersize',d.msize);
    set(gca,'fontsize',d.fsize)
    xlabel('Growth parameter r','fontsize',d.fsize)
    ylabel('x','fontsize',d.fsize)
    title('May Bifurcations Logistic map','fontsize',d.fsize)
    xlim( [xlimits(1),xlimits(2)]);
    ylim( [ylimits(1),ylimits(2)]);
    axis manual
    box on;
    grid on;
end

%%

%May's logistic function x = r * x * (1-x)
function [x,rr] = may( r, N )

%Initialise output array x
x = zeros( N, length(r) );
z = rand( 1, length(r) );

%Step through logistic function, determining new rows of the x positions
%in an iterative fashion

%Initialization
for n=1:N
    z = r.*z.*( -z + 1 );
end

%Determine new 'orbit' positions
for n=1:N
    z = r.*z.*( -z + 1 );
    x(n,:) = z;
end

%Replicate row vector of r values
rr = repmat( r, [N,1]);

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
%
% LAST UPDATED by Andy French 14 September 2008

function interp_colormap( N )

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code